{% extends "base.html" %}

{% block extrahead %}
  <link rel="preconnect" href="https://fonts.loli.net" crossorigin>
  <link href="https://fonts.loli.net/css2?family=JetBrains+Mono:wght@400;700&display=swap" rel="stylesheet">
{% endblock %}
